@@ -231,11 +231,13 @@ sub compile {
     my $header = delete $self->{header};
     my $footer = delete $self->{footer};
 
-    if($header) {
-        substr $input, 0, 0, $self->_cat_files($header);
-    }
-    if($footer) {
-        $input .= $self->_cat_files($footer);
+    if(!$args{from_include}) {
+        if($header) {
+            substr $input, 0, 0, $self->_cat_files($header);
+        }
+        if($footer) {
+            $input .= $self->_cat_files($footer);
+        }
     }
 
     my @code; # main code
@@ -244,7 +246,7 @@ sub compile {
         print STDERR p($ast) if _DUMP_AST;
         @code = (
             $self->opcode(set_opinfo => undef, file => $self->current_file, line => 1),
-            $self->_compile_ast($ast),
+            $self->compile_ast($ast),
             $self->opcode('end'),
         );
     }
@@ -296,6 +298,18 @@ sub opcode { # build an opcode
            ];
 }
 
+sub push_expr {
+    my($self, $node) = @_;
+
+    my $list_op = $node->arity eq 'range';
+    my @code = ($self->compile_ast($node));
+    if(not $list_op) {
+        push @code, $self->opcode('push');
+    }
+    return @code;
+}
+
+
 sub _cat_files {
     my($self, $files) = @_;
     my $engine = $self->engine || $self->_error("No Xslate engine which header/footer requires");
@@ -310,19 +324,14 @@ sub _cat_files {
 
 our $_lv = -1;
 
-sub _compile_ast {
+sub compile_ast {
     my($self, $ast) = @_;
     return if not defined $ast;
 
-    Carp::confess("Oops: Not an ARRAY reference: " . p($ast)) if ref($ast) ne 'ARRAY';
-
-    # TODO
-    # $self->_optimize_ast($ast) if $self->optimize;
-
     local $_lv = $_lv + 1 if _DUMP_GEN;
 
     my @code;
-    foreach my $node(@{$ast}) {
+    foreach my $node(ref($ast) eq 'ARRAY' ? @{$ast} : $ast) {
         blessed($node) or Carp::confess("Oops: Not a node object: " . p($node));
 
         printf STDERR "%s"."generate %s (%s)\n", "." x $_lv, $node->arity, $node->id if _DUMP_GEN;
@@ -336,13 +345,6 @@ sub _compile_ast {
     return @code;
 }
 
-
-sub _expr {
-    my($self, $node) = @_;
-    my @ast = ($node);
-    return $self->_compile_ast(\@ast);
-}
-
 sub _process_cascade {
     my($self, $cascade, $args, $main_code) = @_;
     my $engine = $self->engine
@@ -405,6 +407,9 @@ sub _process_cascade {
                     push @{$macro->{body}}, $c;
                 }
             }
+            elsif($c->[0] eq 'depend') {
+                $self->requires($c->[1]);
+            }
         }
         $self->requires($fullpath);
         $self->_process_cascade_file($cfile, $base_code);
@@ -577,12 +582,12 @@ sub _generate_command {
                 ? 'print_raw'  # mark_raw, raw
                 : 'print';     # html
             push @code,
-                $self->_expr($arg->second->[0]),
+                $self->compile_ast($arg->second->[0]),
                 $self->opcode( $command => undef, symbol => $filter );
         }
         else {
             push @code,
-                $self->_expr($arg),
+                $self->compile_ast($arg),
                 $self->opcode( $proc => undef, line => $node->line );
         }
     }
@@ -617,7 +622,7 @@ sub _generate_cascade {
 
 sub _compile_block {
     my($self, $block) = @_;
-    my @block_code = $self->_compile_ast($block);
+    my @block_code = $self->compile_ast($block);
 
     foreach my $op(@block_code) {
         if($op->[0] eq 'pushmark') {
@@ -644,7 +649,7 @@ sub _generate_for {
     local $self->{lvar}  = { %{$self->lvar} }; # new scope
     local $self->{const} = [];                 # new scope
 
-    my @code = $self->_expr($expr);
+    my @code = $self->compile_ast($expr);
 
     my($iter_var) = @{$vars};
     my $lvar_id   = $self->lvar_id;
@@ -682,7 +687,7 @@ sub _generate_while {
     local $self->{lvar}  = { %{$self->lvar}  }; # new scope
     local $self->{const} = [ @{$self->const} ]; # new scope
 
-    my @code = $self->_expr($expr);
+    my @code = $self->compile_ast($expr);
 
     my($iter_var) = @{$vars};
     my($lvar_id, $lvar_name);
@@ -728,7 +733,7 @@ sub _generate_proc { # definition of macro, block, before, around, after
     my %macro = (
         name      => $name,
         nargs     => $arg_ix,
-        body      => [ $opinfo, $self->_compile_ast($block) ],
+        body      => [ $opinfo, $self->compile_ast($block) ],
         line      => $opinfo->[2],
         file      => $opinfo->[3],
         outer     => $lvar_used,
@@ -755,7 +760,7 @@ sub _generate_lambda {
     my($self, $node) = @_;
 
     my $macro = $node->first;
-    $self->_compile_ast([$macro]);
+    $self->compile_ast($macro);
     return $self->opcode( fetch_symbol => $macro->first->id, line => $node->line );
 }
 
@@ -796,23 +801,24 @@ sub _generate_if {
 
     local $self->{lvar}  = { %{$self->lvar}  }; # new scope
     local $self->{const} = [ @{$self->const} ]; # new scope
-    my @cond  = $self->_expr($expr);
+    my @cond  = $self->compile_ast($expr);
 
     my @then = do {
         local $self->{lvar}  = { %{$self->lvar}  }; # new scope
         local $self->{const} = [ @{$self->const} ]; # new scope
-        $self->_compile_ast($second);
+        $self->compile_ast($second);
     };
 
     my @else = do {
         local $self->{lvar}  = { %{$self->lvar}  }; # new scope
         local $self->{const} = [ @{$self->const} ]; # new scope
-        $self->_compile_ast($third);
+        $self->compile_ast($third);
     };
 
     if($OPTIMIZE) {
         if($self->_code_is_literal(@cond)) {
-            if($cond[0][1]) {
+            my $value = $cond[0][1];
+            if($cond_true eq 'and' ? $value : !$value) {
                 return @then;
             }
             else {
@@ -858,7 +864,7 @@ sub _generate_given {
     local $self->{lvar}  = { %{$self->lvar}  }; # new scope
     local $self->{const} = [ @{$self->const} ]; # new scope
 
-    my @code = $self->_expr($expr);
+    my @code = $self->compile_ast($expr);
 
     my($lvar)     = @{$vars};
     my $lvar_id   = $self->lvar_id;
@@ -868,7 +874,7 @@ sub _generate_given {
 
     local $self->{lvar_id} = $self->lvar_use(1); # topic variable
     push @code, $self->opcode( save_to_lvar => $lvar_id, symbol => $lvar ),
-        $self->_compile_ast($block);
+        $self->compile_ast($block);
 
     return @code;
 }
@@ -912,7 +918,7 @@ sub _generate_composer {
 
     return
         $self->opcode( pushmark => undef, comment => $type ),
-        (map{ $self->_expr($_), ['push'] } @{$list}),
+        (map{ $self->push_expr($_) } @{$list}),
         $self->opcode($type),
     ;
 }
@@ -923,7 +929,7 @@ sub _generate_unary {
     my $id = $node->id;
     if(exists $unary{$id}) {
         my @code = (
-            $self->_expr($node->first),
+            $self->compile_ast($node->first),
             $self->opcode( $unary{$id} )
         );
         if( $OPTIMIZE and $self->_code_is_literal($code[0]) ) {
@@ -939,7 +945,7 @@ sub _generate_unary {
 sub _generate_field {
     my($self, $node) = @_;
 
-    my @lhs   = $self->_expr($node->first);
+    my @lhs   = $self->compile_ast($node->first);
     my $field = $node->second;
 
     # $foo.field
@@ -952,7 +958,7 @@ sub _generate_field {
     # $foo[expression]
     else {
         local $self->{lvar_id} = $self->lvar_use(1);
-        my @rhs = $self->_expr($field);
+        my @rhs = $self->compile_ast($field);
         if($OPTIMIZE and $self->_code_is_literal(@rhs)) {
             return
                 @lhs,
@@ -972,12 +978,12 @@ sub _generate_field {
 sub _generate_binary {
     my($self, $node) = @_;
 
-    my @lhs = $self->_expr($node->first);
+    my @lhs = $self->compile_ast($node->first);
 
     my $id = $node->id;
     if(exists $binary{$id}) {
         local $self->{lvar_id} = $self->lvar_use(1);
-        my @rhs = $self->_expr($node->second);
+        my @rhs = $self->compile_ast($node->second);
         my @code = (
             @lhs,
             $self->opcode( save_to_lvar => $self->lvar_id ),
@@ -1005,7 +1011,7 @@ sub _generate_binary {
         return @code;
     }
     elsif(exists $logical_binary{$id}) {
-        my @rhs = $self->_expr($node->second);
+        my @rhs = $self->compile_ast($node->second);
         return
             @lhs,
             $self->opcode( $logical_binary{$id} => scalar(@rhs) + 1, symbol => $node ),
@@ -1015,6 +1021,22 @@ sub _generate_binary {
     $self->_error("Binary operator $id is not implemented", $node);
 }
 
+sub _generate_range {
+    my($self, $node) = @_;
+
+    my @lhs  = $self->compile_ast($node->first);
+
+    local $self->{lvar_id} = $self->lvar_use(1);
+    my @rhs = $self->compile_ast($node->second);
+    return(
+        @lhs,
+        $self->opcode( save_to_lvar => $self->lvar_id ),
+        @rhs,
+        $self->opcode( load_lvar_to_sb => $self->lvar_id ),
+        $self->opcode( 'range' ),
+    );
+}
+
 sub _generate_methodcall {
     my($self, $node) = @_;
 
@@ -1022,9 +1044,8 @@ sub _generate_methodcall {
     my $method = $node->second->value;
     return (
         $self->opcode( pushmark => undef, comment => $method ),
-        $self->_expr($node->first),
-        $self->opcode( 'push' ),
-        (map { $self->_expr($_), $self->opcode('push') } @{$args}),
+        $self->push_expr($node->first),
+        (map { $self->push_expr($_) } @{$args}),
         $self->opcode( methodcall_s => $method, line => $node->line ),
     );
 }
@@ -1038,13 +1059,13 @@ sub _generate_call {
         if(@{$args} != 1) {
             $self->_error("Wrong number of arguments for $callable", $node);
         }
-        return $self->_expr($args->[0]), [ $intern => undef, $node->line ];
+        return $self->compile_ast($args->[0]), [ $intern => undef, $node->line ];
     }
 
     return(
         $self->opcode( pushmark => undef, comment => $callable->id ),
-        (map { $self->_expr($_), $self->opcode( 'push' ) } @{$args}),
-        $self->_expr($callable),
+        (map { $self->push_expr($_) } @{$args}),
+        $self->compile_ast($callable),
         $self->opcode( 'funcall' )
     );
 }
@@ -1089,7 +1110,7 @@ sub _generate_assign {
         $self->_error("Assignment ($node) is not supported", $node);
     }
 
-    my @expr = $self->_expr($rhs);
+    my @expr = $self->compile_ast($rhs);
 
     if($is_decl) {
         $lvar->{$lvar_name} = $self->lvar_id;
@@ -1110,7 +1131,7 @@ sub _generate_constant {
     my $lhs     = $node->first;
     my $rhs     = $node->second;
 
-    my @expr = $self->_expr($rhs);
+    my @expr = $self->compile_ast($rhs);
 
     my $lvar            = $self->lvar;
     my $lvar_id         = $self->lvar_id;
@@ -1131,7 +1152,6 @@ sub _generate_constant {
         $self->opcode( save_to_lvar => $lvar_id, symbol => $lhs, comment => $node->id);
 }
 
-
 sub _localize_vars {
     my($self, $vars) = @_;
     my @localize;
@@ -1141,7 +1161,7 @@ sub _localize_vars {
             $self->_error("You must pass a simple name to localize variables", $key);
         }
         push @localize,
-            $self->_expr($expr),
+            $self->compile_ast($expr),
             $self->opcode( localize_s => $key->value, symbol => $key );
     }
     return @localize;
@@ -1179,7 +1199,7 @@ sub _fold_constants {
     my $result = eval {
         my @tmp_code = (@{$code}, $self->opcode('print_raw'), $self->opcode('end'));
         $engine->_assemble(\@tmp_code, '<string>', undef, undef, undef);
-        $engine->render(undef);
+        $engine->render('<string>');
     };
     if($@) {
         Carp::carp("Oops: constant folding failed (ignored): $@");
@@ -1355,7 +1375,7 @@ __END__
 
 =head1 NAME
 
-Text::Xslate::Compiler - The Xslate compiler
+Text::Xslate::Compiler - An Xslate compiler to generate intermediate code
 
 =head1 DESCRIPTION
 
