@@ -6,7 +6,6 @@ use Text::Xslate::Util qw(
     value_to_literal
 );
 
-
 use Carp ();
 use Scalar::Util ();
 
@@ -65,7 +64,6 @@ sub opcode_to_perlcode {
     my ( $self, $opcode ) = @_;
 
     my $perlcode = $self->opcode_to_perlcode_string( $opcode );
-    print STDERR $perlcode, "\n" if _DUMP_PP;
     return Text::Xslate::PP::Booster->compile($perlcode);
 }
 
@@ -99,6 +97,7 @@ CODE
     $perlcode .= join( '', grep { defined } @{ $self->{lines} } );
     $perlcode .= "\n" . '    return $output;' . "\n}";
 
+    print STDERR $perlcode, "\n" if _DUMP_PP;
     return $perlcode;
 }
 
@@ -309,7 +308,7 @@ $CODE_MANIP{ 'include' } = sub {
 # include
 {
     $st->{pc} = %2$s; # for error messages
-    my $st2 = Text::Xslate::PP::tx_load_template( $st->engine, %1$s );
+    my $st2 = Text::Xslate::PP::tx_load_template( $st->engine, %1$s, 1 );
     $output .= Text::Xslate::PP::tx_execute( $st2, $vars );
 }
 CODE
@@ -561,6 +560,11 @@ $CODE_MANIP{ 'scmp' } = sub {
     $self->sa( sprintf( '( %s cmp %s )', _rm_tailed_lf( $self->sb() ), _rm_tailed_lf( $self->sa() ) ) );
 };
 
+$CODE_MANIP{ 'range' } = sub {
+    my ( $self, $arg, $line ) = @_;
+    push @{ $self->SP->[ -1 ] }, sprintf('( %s .. %s )',
+        _rm_tailed_lf( $self->sb() ), _rm_tailed_lf( $self->sa() ) );
+};
 
 $CODE_MANIP{ 'macrocall' } = sub {
     my ( $self, $arg, $line ) = @_;
@@ -1156,7 +1160,7 @@ __END__
 
 =head1 NAME
 
-Text::Xslate::Compiler::PPBooster::CodeGenerator - PP::Booster code generator
+Text::Xslate::PP::Compiler::CodeGenerator - An Xslate code generator for PP::Booster
 
 =head1 DESCRIPTION
 
